<?php
/*
 *  registering the widgetizable sidebar
*/
if ( function_exists('register_sidebar') ) {
    register_sidebar(array(
            'before_widget' => '<div class="widget">',
            'after_widget' => '</div>',
            'before_title' => '<h2 class="widgettitle">',
            'after_title' => '</h2>',
    ));
}
/*
 *  loading each JS-file
*/
function loadScripts() {
    wp_enqueue_script('jquery');
    wp_enqueue_script( 'jqueryuicustom', get_bloginfo('template_directory').'/js/jquery-ui-1.7.2.custom.min.js', array( 'jquery' ) );
    wp_enqueue_script( 'cufon', get_bloginfo('template_directory').'/js/cufon-yui.js', array( 'jquery' ) );
    wp_enqueue_script( 'fhMain', get_bloginfo('template_directory').'/js/main.js', array( 'jquery' ) );
}
/*
 *  defining a new excerpt length
*/
function new_excerpt_length($length) {
    return 30;
}
add_filter('excerpt_length', 'new_excerpt_length');

/*
 *  defining a new excerpt 'more'-message
*/
function new_excerpt_more($more) {
    return str_replace('[...]','...', $more);
}
add_filter('excerpt_more', 'new_excerpt_more');

/*
 * 1) show advertisement banner
 * 2) social icons
*/
function headerAdvertisement() {
    if(get_option('fh_adv')) {?>
<div id="adv_468"><a href="<?php echo get_option('fh_adv_link') ?>"><img src="<?php echo get_option('fh_adv_img') ?>" alt="advertisement"/><span class="hidden">&nbsp;</span></a></div>
        <?php }else {?>
<div class="share">
    <ul>
        <li><a href="http://www.stumbleupon.com/submit?url=<?php echo urlencode(get_option('home')); ?>" class="stumbleupon"> <span class="hidden">stumbleupon</span> </a> </li>
        <li><a href="http://www.technorati.com/faves?add=<?php echo urlencode(get_option('home')); ?>" class="technorati"> <span class="hidden">technorati</span> </a> </li>
        <li><a href="http://twitter.com/home?status=<?php bloginfo('name')?> - <?php bloginfo('description'); ?> (<?php echo getTinyUrl(get_option('home'));?>)" class="twitter"> <span class="hidden">twitter</span> </a> </li>
        <li><a href="http://del.icio.us/post?url=<?php echo urlencode(get_option('home')); ?>" class="delicious"> <span class="hidden">delicious</span> </a> </li>
        <li><a href="http://www.facebook.com/share.php?u=<?php echo urlencode(get_option('home')); ?>" class="facebook"> <span class="hidden">facebook</span> </a> </li>
        <li><a href="http://digg.com/submit?phase=2&amp;url=<?php echo urlencode(get_option('home')); ?>&amp;title=<?php urlencode(bloginfo('name')); ?>-<?php urlencode(bloginfo('description')); ?>" class="digg"> <span class="hidden">digg</span> </a> </li>
    </ul>
</div>
        <?php }
}
/*
 * returns the logo
*/
function getLogo() {
    return '<img src="'. get_option('fh_logo') .'" alt="'.get_bloginfo('name').' - Home" />';
}
/*
 * some predefined CSS options in the monk panel
*/
function getCSS() {
    $monk_css .= 'a{color:'. get_option('fh_colors_a').'}';
    $monk_css .= 'a:hover{color:'. get_option('fh_colors_a_hover').'}';
    $monk_css .= 'body{background-color:'. get_option('fh_colors_body').'}';
    $monk_css .= '#footer_bg{background-color:'. get_option('fh_colors_footer').'}';
    $monk_css .= get_option('fh_custom_css');
    if (strlen($monk_css) > 0) {
        return  '<style type="text/css">'.addslashes($monk_css)."</style>";
    }
    else {
        return;
    }
}
/*
 * creating a tinyurl based on a give url
*/
function getTinyUrl($url) {
    $tinyurl = file_get_contents("http://tinyurl.com/api-create.php?url=".$url);
    return $tinyurl;
}
/*
 * get the feed based on a feedburner account or if none is set, the default Wordpress RSS
*/
function getFeed() {
    $feedburner_account = get_option('fh_rss');
    $pos = strpos($feedburner_account, "http");

    if ($pos === false) {
        $whaturl="http://feeds.feedburner.com/$feedburner_account";
        return $whaturl;
    }
    else {
        return $feedburner_account;
    }
}
/*
 * get the feedcount based on the feedburner account
*/
function getFeedCount() {
    $feedburner_account = get_option('fh_rss');

    if ($feedburner_account == "")
        return "no";
    try {
        $whaturl="https://feedburner.google.com/api/awareness/1.0/GetFeedData?uri=$feedburner_account";
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_URL, $whaturl);
        $data = curl_exec($ch);
        curl_close($ch);
        $xml = new SimpleXMLElement($data);
        $fb = $xml->feed->entry['circulation'];
        return $fb;
    }
    catch (Exception $e) {
        return $e->getCode();
    }
}

function get_pagelink_by_title($page_title) {
    global $wpdb;
    $page = $wpdb->get_var( $wpdb->prepare( "SELECT ID FROM $wpdb->posts WHERE post_title = %s AND post_type='page'", $page_title ));
    if ($page)
        return get_page_link($page);
    return null;
}

function trimText($text,$max_char) {
    if(strlen($text) > $max_char) {
        $text = substr($text, 0, $max_char-3).'...';
    }
    return $text;
}

function getParentCategories() {

    global $wpdb;
    $sql = "SELECT term_id as id, name, slug FROM $wpdb->prefix"."terms where term_id in (SELECT term_id FROM $wpdb->prefix"."term_taxonomy where parent = 0 and taxonomy = 'category' and term_id != 1) order by name asc";
    $parents = $wpdb->get_results( $sql );
    return $parents;
}

function getChildCategories($id) {
    global $wpdb;
    $sql = "SELECT term_id as id, name, slug FROM $wpdb->prefix"."terms where term_id in (SELECT term_id FROM $wpdb->prefix"."term_taxonomy where parent = $id and taxonomy = 'category')  order by name asc";
    $children = $wpdb->get_results( $sql );
    return $children;
}

function monk_next_posts($label='Next Page',$max_page=0, $class='') {
    global $paged;
    if ( (!is_single())	&& ($paged > 1) ) {
        echo '<a  class="'. $class . '" href="';
        previous_posts();
        echo '">'. preg_replace('/&([^#])(?![a-z]{1,8};)/', '&#038;$1', $label) .'</a>';
    }
}
function monk_previous_posts($label='Previous Page', $max_page=0, $class='') {
    global $paged, $wp_query;
    if ( !$max_page ) {
        $max_page = $wp_query->max_num_pages;

        if ( (! is_single()) && (empty($paged) || $nextpage <= $max_page) ) {
            echo '<a  class="'. $class . '" href="';
            next_posts($max_page);
            echo '">'. preg_replace('/&([^#])(?![a-z]{1,8};)/', '&#038;$1', $label) .'</a>';
        }
    }
}

//comment functions
include_once TEMPLATEPATH.'/functions/comments.php';
//widgets
include_once TEMPLATEPATH.'/functions/widgets.php';
//monkpanel
include_once TEMPLATEPATH.'/functions/monkpanel/functions.php';
?>